# frozen_string_literal: true

class ComplexSubject < ActiveRecord::Base
  has_many :complex_sessions, foreign_key: 'parent_source_identifier', primary_key: 'source_identifier', dependent: :destroy

  attribute :source_identifier, :string, default: -> { SecureRandom.uuid }

  validates :source_identifier, presence: true, uniqueness: true
  validates :parent_source_identifier, presence: true
  validates :subject_title, presence: { message: "Subject folder name can't be blank" }
  validates :subject_title, uniqueness: { scope: :parent_source_identifier,  message: "Subject folder name is already taken" }
  validates :subject_title, format: { with: /\A[a-zA-Z0-9][a-zA-Z0-9\-\. ]*\z/, message: "Folder name not valid. Folder name should be a combination of only lowercase letters, numbers, hyphen, dot, and space. The name and should start with a letter or number" }

  after_save :save_work_meta_json_file_to_s3, :reindex_crc_dataset
  before_validation :fix_subject_title

  def crc_dataset
    CrcDataset.where(source: parent_source_identifier).first
  end

  def file_sets
    FileSet.where(for_complex_identifier: source_identifier)
  end

  def file_sets_fast_load_metadata
    FileSetFastLoadMetadata.where(work_source_identifier: source_identifier)
  end

  private

  def fix_subject_title
    self.subject_title = self.subject_title.downcase.gsub(/^[^a-z]+|[^\w\s.-]/i, '')
  end

  def reindex_crc_dataset
    Hyrax::ReindexCrcDatasetJob.perform_later(crc_dataset.id)
  end

  def save_work_meta_json_file_to_s3
    s3 = S3StorageService.new
    s3.init_client
    bucket_name = s3.sanitise_name(crc_dataset.id)
    s3.create_bucket(bucket_name) unless s3.bucket_exists?(bucket_name)
    s3.add_content(bucket_name, "#{subject_title}/metadata.json", to_json)
  end
end
